import { unsafe_mutateSubgraph as mutateSubgraph, unsafe_MutatorFlow as MutatorFlow } from "@typespec/compiler/experimental";
import { $ } from "@typespec/compiler/experimental/typekit";

/**
 * Prepares operation for client representation. This includes adding an options bag for optional parameters.
 * @param operation operation to be prepared
 * @returns the prepared operation
 */
export function prepareOperation(operation) {
  return mutateSubgraph($.program, [httpParamsMutator], operation).type;
}

/**
 * Mutates anonymous types to be named.
 */
const anonymousMutatorRecord = {
  filter(t) {
    return MutatorFlow.MutateAndRecur;
  },
  mutate(t, clone) {
    if (!clone.name) {
      clone.name = $.type.getPlausibleName(clone);
    }
  }
};
export const anonymousMutator = {
  name: "Anonymous types",
  Model: anonymousMutatorRecord,
  Union: anonymousMutatorRecord
};

/**
 * Mutates the operation so that the parameters model is split into required and optional parameters.
 */
export const httpParamsMutator = {
  name: "Http parameters",
  Operation: {
    filter() {
      return MutatorFlow.DoNotRecur;
    },
    mutate(o, clone, _program, realm) {
      const httpOperation = $.httpOperation.get(o);
      const returnType = $.httpOperation.getReturnType(httpOperation);
      clone.returnType = returnType;
      const params = $.httpRequest.getParameters(httpOperation, ["query", "header", "path", "body", "contentType"]);
      if (!params) {
        return;
      }
      clone.parameters = params;
      const optionals = [...clone.parameters.properties.values()].filter(p => p.optional).reduce((acc, prop) => {
        acc[prop.name] = prop;
        return acc;
      }, {});
      const optionsBag = $.model.create({
        properties: optionals
      });
      const optionsProp = $.modelProperty.create({
        name: "options",
        type: optionsBag,
        optional: true
      });
      for (const [key, prop] of clone.parameters.properties) {
        if (prop.optional || isConstantHeader(prop)) {
          clone.parameters.properties.delete(key);
        }
      }
      if (Object.keys(optionals).length > 0) {
        clone.parameters.properties.set("options", optionsProp);
      }
    }
  }
};
export function isConstantHeader(modelProperty) {
  if (!$.modelProperty.isHttpHeader(modelProperty)) {
    return false;
  }
  if ("value" in modelProperty.type && modelProperty.type.value !== undefined) {
    return true;
  }
  return false;
}
//# sourceMappingURL=operations.js.map